/*
 * Copyright © 2011 Linaro Limited
 *
 * This file is part of glcompbench.
 *
 * glcompbench is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * glcompbench is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with glcompbench.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *  Alexandros Frantzis <alexandros.frantzis@linaro.org>
 *  Jesse Barker <jesse.barker@linaro.org>
 */

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/Xrender.h>

#include "composite-window-xrender.h"
#include "log.h"

CompositeWindowXRender::~CompositeWindowXRender()
{
    release_resources();
}

void
CompositeWindowXRender::release_resources()
{
    if (xrender_picture_)
        XRenderFreePicture(xdpy_, xrender_picture_);

    xrender_picture_ = 0;
}

bool
CompositeWindowXRender::ensure_xrender_picture()
{
    if (xrender_picture_)
        return true;

    XWindowAttributes win_attr;

    if (!XGetWindowAttributes(xdpy_, win_, &win_attr)) {
        Log::error("XGetWindowAttributes failed!\n");
        return false;
    }

    /* TODO: should we free this when we are done? */
    XRenderPictFormat *pict_format;
        
    pict_format = XRenderFindVisualFormat(xdpy_, win_attr.visual);
    if (!pict_format) {
        Log::error("XRenderFindVisualFormat failed!\n");
        release_resources();
        return false;
    }

    xrender_picture_ = XRenderCreatePicture(xdpy_, win_, pict_format,
                                            0, NULL);
    if (!xrender_picture_) {
        Log::error("XRenderCreatePicture failed!\n");
        release_resources();
        return false;
    }

    return true;
}

void
CompositeWindowXRender::update_texture()
{
    release_resources();
    ensure_xrender_picture();
}

void
CompositeWindowXRender::update_texture_from_pixmap()
{
    Profiler::ScopedSamples scoped_tfp(CompositeWindow::get_profiler_tfp_pair());
}

